@@ -12,8 +12,8 @@
 import sys
 from os import path
 
-__version__ = '1.0'
-__released__ = '1.0'  # used when Sphinx builds its own docs
+__version__ = '1.0.2'
+__released__ = '1.0.2'  # used when Sphinx builds its own docs
 
 package_dir = path.abspath(path.dirname(__file__))
 
