@@ -5,16 +5,22 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
 import cgi
 import re
-import parser
+import textwrap
 
-from sphinx.util.texescape import tex_hl_escape_map
+try:
+    import parser
+except ImportError:
+    # parser is not available on Jython
+    parser = None
+
+from sphinx.util.texescape import tex_hl_escape_map_old, tex_hl_escape_map_new
 
 try:
     import pygments
@@ -28,8 +34,11 @@ try:
     from pygments.styles import get_style_by_name
     from pygments.styles.friendly import FriendlyStyle
     from pygments.token import Generic, Comment, Number
+    from pygments.util import ClassNotFound
 except ImportError:
     pygments = None
+    lexers = None
+    HtmlFormatter = LatexFormatter = None
 else:
     class SphinxStyle(Style):
         """
@@ -47,12 +56,15 @@ else:
             Number: '#208050',
         })
 
+    class NoneStyle(Style):
+        """Style without any styling."""
+
     lexers = dict(
         none = TextLexer(),
         python = PythonLexer(),
         pycon = PythonConsoleLexer(),
-        # the python3 option exists as of Pygments 0.12, but it doesn't
-        # do any harm in previous versions
+        # the python3 option exists as of Pygments 1.0,
+        # but it doesn't do any harm in previous versions
         pycon3 = PythonConsoleLexer(python3=True),
         rest = RstLexer(),
         c = CLexer(),
@@ -72,6 +84,7 @@ _LATEX_STYLES = r'''
 \newcommand\PYGZrb{]}
 '''
 
+doctestopt_re = re.compile(r'#\s*doctest:.+$', re.MULTILINE)
 
 parsing_exceptions = (SyntaxError, UnicodeEncodeError)
 if sys.version_info < (2, 5):
@@ -81,22 +94,35 @@ if sys.version_info < (2, 5):
 
 
 class PygmentsBridge(object):
-    def __init__(self, dest='html', stylename='sphinx'):
+    # Set these attributes if you want to have different Pygments formatters
+    # than the default ones.
+    html_formatter = HtmlFormatter
+    latex_formatter = LatexFormatter
+
+    def __init__(self, dest='html', stylename='sphinx',
+                 trim_doctest_flags=False):
         self.dest = dest
         if not pygments:
             return
-        if stylename == 'sphinx':
+        if stylename is None or stylename == 'sphinx':
             style = SphinxStyle
+        elif stylename == 'none':
+            style = NoneStyle
         elif '.' in stylename:
             module, stylename = stylename.rsplit('.', 1)
-            style = getattr(__import__(module, None, None, ['']), stylename)
+            style = getattr(__import__(module, None, None, ['__name__']),
+                            stylename)
         else:
             style = get_style_by_name(stylename)
-        self.hfmter = {False: HtmlFormatter(style=style),
-                       True: HtmlFormatter(style=style, linenos=True)}
-        self.lfmter = {False: LatexFormatter(style=style, commandprefix='PYG'),
-                       True: LatexFormatter(style=style, linenos=True,
-                                            commandprefix='PYG')}
+        self.trim_doctest_flags = trim_doctest_flags
+        if dest == 'html':
+            self.fmter = {False: self.html_formatter(style=style),
+                          True: self.html_formatter(style=style, linenos=True)}
+        else:
+            self.fmter = {False: self.latex_formatter(style=style,
+                                                      commandprefix='PYG'),
+                          True: self.latex_formatter(style=style, linenos=True,
+                                                     commandprefix='PYG')}
 
     def unhighlighted(self, source):
         if self.dest == 'html':
@@ -105,7 +131,7 @@ class PygmentsBridge(object):
             # first, escape highlighting characters like Pygments does
             source = source.translate(escape_hl_chars)
             # then, escape all characters nonrepresentable in LaTeX
-            source = source.translate(tex_hl_escape_map)
+            source = source.translate(tex_hl_escape_map_old)
             return '\\begin{Verbatim}[commandchars=@\\[\\]]\n' + \
                    source + '\\end{Verbatim}\n'
 
@@ -113,6 +139,10 @@ class PygmentsBridge(object):
         # Make sure it ends in a newline
         src += '\n'
 
+        # Ignore consistent indentation.
+        if src.lstrip('\n').startswith(' '):
+            src = textwrap.dedent(src)
+
         # Replace "..." by a mark which is also a valid python expression
         # (Note, the highlighter gets the original source, this is only done
         #  to allow "..." in code and still highlight it as Python code.)
@@ -134,6 +164,9 @@ class PygmentsBridge(object):
             # just replace all non-ASCII characters.
             src = src.encode('ascii', 'replace')
 
+        if parser is None:
+            return True
+
         try:
             parser.suite(src)
         except parsing_exceptions:
@@ -141,9 +174,13 @@ class PygmentsBridge(object):
         else:
             return True
 
-    def highlight_block(self, source, lang, linenos=False):
+    def highlight_block(self, source, lang, linenos=False, warn=None):
+        if isinstance(source, str):
+            source = source.decode()
         if not pygments:
             return self.unhighlighted(source)
+
+        # find out which lexer to use
         if lang in ('py', 'python'):
             if source.startswith('>>>'):
                 # interactive session
@@ -166,14 +203,31 @@ class PygmentsBridge(object):
             if lang in lexers:
                 lexer = lexers[lang]
             else:
-                lexer = lexers[lang] = get_lexer_by_name(lang)
-                lexer.add_filter('raiseonerror')
+                try:
+                    lexer = lexers[lang] = get_lexer_by_name(lang)
+                except ClassNotFound:
+                    if warn:
+                        warn('Pygments lexer name %s is not known' % lang)
+                        return self.unhighlighted(source)
+                    else:
+                        raise
+                else:
+                    lexer.add_filter('raiseonerror')
+
+        # trim doctest options if wanted
+        if isinstance(lexer, PythonConsoleLexer) and self.trim_doctest_flags:
+            source = doctestopt_re.sub('', source)
+
+        # highlight via Pygments
         try:
             if self.dest == 'html':
-                return highlight(source, lexer, self.hfmter[bool(linenos)])
+                return highlight(source, lexer, self.fmter[bool(linenos)])
             else:
-                hlsource = highlight(source, lexer, self.lfmter[bool(linenos)])
-                return hlsource.translate(tex_hl_escape_map)
+                hlsource = highlight(source, lexer, self.fmter[bool(linenos)])
+                if hlsource.startswith(r'\begin{Verbatim}[commandchars=\\\{\}'):
+                    # Pygments >= 1.2
+                    return hlsource.translate(tex_hl_escape_map_new)
+                return hlsource.translate(tex_hl_escape_map_old)
         except ErrorToken:
             # this is most probably not the selected language,
             # so let it pass unhighlighted
@@ -186,9 +240,9 @@ class PygmentsBridge(object):
             # no HTML styles needed
             return ''
         if self.dest == 'html':
-            return self.hfmter[0].get_style_defs()
+            return self.fmter[0].get_style_defs('.highlight')
         else:
-            styledefs = self.lfmter[0].get_style_defs()
+            styledefs = self.fmter[0].get_style_defs()
             # workaround for Pygments < 0.12
             if styledefs.startswith('\\newcommand\\at{@}'):
                 styledefs += _LATEX_STYLES
